/**
 * Extend the basic ActorSheet with some very simple modifications
 * @extends {ActorSheet}
 */

import { PegasusUtility } from "./pegasus-utility.js";
import { PegasusItemSheet } from "./pegasus-item-sheet.js";
import { PegasusRollDialog } from "./pegasus-roll-dialog.js";

/* -------------------------------------------- */
export class PegasusActorSheet extends ActorSheet {

  /** @override */
  static get defaultOptions() {

    return mergeObject(super.defaultOptions, {
      classes: ["fvtt-pegasus-rpg", "sheet", "actor"],
      template: "systems/fvtt-pegasus-rpg/templates/actor-sheet.html",
      width: 640,
      height: 720,
      tabs: [{ navSelector: ".sheet-tabs", contentSelector: ".sheet-body", initial: "stats" }],
      dragDrop: [{ dragSelector: ".item-list .item", dropSelector: null }],
      editScore: false
    });
  }

  /* -------------------------------------------- */
  async getData() {
    const objectData = PegasusUtility.data(this.object);
    
    let actorData = duplicate(PegasusUtility.templateData(this.object));

    let formData = {
      title: this.title,
      id: objectData.id,
      type: objectData.type,
      img: objectData.img,
      name: objectData.name,
      editable: this.isEditable,
      cssClass: this.isEditable ? "editable" : "locked",
      data: actorData,
      effects: this.object.effects.map(e => foundry.utils.deepClone(e.data)),
      limited: this.object.limited,
      specs: this.actor.getSpecs( ),
      optionsDiceList: PegasusUtility.getOptionsDiceList(),
      optionsLevel: PegasusUtility.getOptionsLevel(),
      weapons: this.actor.checkAndPrepareWeapons( duplicate(this.actor.getWeapons()) ),
      armors: this.actor.checkAndPrepareArmors( duplicate(this.actor.getArmors())),
      shields: duplicate(this.actor.getShields()),
      equipments: duplicate(this.actor.getEquipments()),
      equipments: duplicate(this.actor.getEquipmentsOnly()),
      perks: duplicate(this.actor.getPerks()),
      abilities: duplicate(this.actor.getAbilities()),
      activePerks: duplicate(this.actor.getActivePerks()),
      powers: duplicate(this.actor.getPowers()),
      subActors: duplicate(this.actor.getSubActors()),
      race: duplicate(this.actor.getRace()),
      role: duplicate(this.actor.getRole()),
      effects: duplicate(this.actor.getEffects()),
      moneys: duplicate(this.actor.getMoneys()),
      options: this.options,
      owner: this.document.isOwner,
      editScore: this.options.editScore,
      isGM: game.user.isGM
    }
    this.formData = formData;

    console.log("PC : ", formData, this.object);
    return formData;
  }

  /* -------------------------------------------- */
  async openGenericRoll() {
    let rollData = PegasusUtility.getBasicRollData()
    rollData.alias = "Dice Pool Roll", 
    rollData.mode  = "generic"
    rollData.title = `Dice Pool Roll`;
    
    let rollDialog = await PegasusRollDialog.create( this.actor, rollData);
    rollDialog.render( true );
  }

  /* -------------------------------------------- */
  /** @override */
  activateListeners(html) {
    super.activateListeners(html);

    // Everything below here is only needed if the sheet is editable
    if (!this.options.editable) return;
    
    html.bind("keydown", function(e) { // Ignore Enter in actores sheet
      if (e.keyCode === 13) return false;
    });  

    // Update Inventory Item
    html.find('.item-edit').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      const item = this.actor.items.get( itemId );
      item.sheet.render(true);
    });
    // Delete Inventory Item
    html.find('.item-delete').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      PegasusUtility.confirmDelete(this, li);
    });
    
    html.find('.spec-group-activate').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      this.actor.specPowerActivate( itemId)
    });      
    html.find('.spec-group-deactivate').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      this.actor.specPowerDeactivate( itemId)
    });      
    
    html.find('.effect-used').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      this.actor.perkEffectUsed( itemId)
    });      

    html.find('.perk-status').change(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      this.actor.updatePerkStatus( itemId, ev.currentTarget.value)
    });
    html.find('.power-cost-spent').change(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let itemId = li.data("item-id");
      this.actor.updatePowerSpentCost( itemId, ev.currentTarget.value)
    });
    html.find('.perk-used').change(ev => {
      const li = $(ev.currentTarget).parents(".item")
      let itemId = li.data("item-id")
      let index = Number($(ev.currentTarget).data("use-index") )
      this.actor.updatePerkUsed( itemId, index, ev.currentTarget.checked )
    });
    
    html.find('.subactor-edit').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let actorId = li.data("actor-id");
      let actor = game.actors.get( actorId );
      actor.sheet.render(true);
    });
    
    html.find('.subactor-delete').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let actorId = li.data("actor-id");
      this.actor.delSubActor(actorId);
    });
    
    html.find('.quantity-minus').click(event => {
      const li = $(event.currentTarget).parents(".item");
      this.actor.incDecQuantity( li.data("item-id"), -1 );
    } );
    html.find('.quantity-plus').click(event => {
      const li = $(event.currentTarget).parents(".item");
      this.actor.incDecQuantity( li.data("item-id"), +1 );
    } );

    html.find('.unarmed-attack').click((event) => {
      this.actor.rollUnarmedAttack();
    });
    html.find('.generic-pool-roll').click((event) => {      
      this.openGenericRoll()
    } );
    html.find('.attack-melee').click((event) => {
      this.actor.rollPool( 'com');
    });
    html.find('.attack-ranged').click((event) => {
      this.actor.rollPool( 'agi');
    });
    html.find('.defense-roll').click((event) => {
      this.actor.rollPool( 'def', true);
    });
    html.find('.damage-melee').click((event) => {
      this.actor.rollPool( 'str');
    });
    html.find('.damage-ranged').click((event) => {
      this.actor.rollPool( 'per');
    });
    html.find('.damage-resistance').click((event) => {
      this.actor.rollPool( 'phy');
    });
            
    html.find('.roll-stat').click((event) => {
      const statId = $(event.currentTarget).data("stat-key");
      this.actor.rollStat(statId);
    });
    html.find('.roll-mr').click((event) => {
      this.actor.rollMR();
    });
    
    html.find('.roll-spec').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const specId = li.data("item-id");
      this.actor.rollSpec(specId);
    });
    html.find('.power-roll').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const powerId = li.data("item-id");
      this.actor.rollPower(powerId);
    });        
    html.find('.weapon-roll').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const weaponId = li.data("item-id");
      this.actor.rollWeapon(weaponId);
    });
    html.find('.armor-roll').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const armorId = li.data("item-id");
      this.actor.rollArmor(armorId);
    });
    
    html.find('.weapon-damage-roll').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const weaponId = li.data("item-id");
      this.actor.rollWeapon(weaponId, true);
    });
    
    html.find('.weapon-damage').click((event) => {
      const li = $(event.currentTarget).parents(".item");
      const weapon = this.actor.getOwnedItem(li.data("item-id"));
      this.actor.rollDamage(weapon, 'damage');
    });
    
    html.find('.lock-unlock-sheet').click((event) => {
      this.options.editScore = !this.options.editScore;
      this.render(true);
    });    
    html.find('.item-link a').click((event) => {
      const itemId = $(event.currentTarget).data("item-id");
      const item = this.actor.getOwnedItem(itemId);
      item.sheet.render(true);
    });    
    html.find('.item-equip').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.actor.equipItem( li.data("item-id") );
      this.render(true);      
    });
    html.find('.power-activate').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.actor.activatePower( li.data("item-id") );
      this.render(true); 
    });
    
    html.find('.change-worstfear').change(ev => {
      this.actor.manageWorstFear( ev.currentTarget.checked )
    });
    html.find('.change-desires').change(ev => {
      this.actor.manageDesires( ev.currentTarget.checked )
    });

    html.find('.update-field').change(ev => {
      const fieldName = $(ev.currentTarget).data("field-name");
      let value = Number(ev.currentTarget.value);
      this.actor.update( { [`${fieldName}`]: value } );
    });    
    html.find('.perk-active').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.actor.activatePerk( li.data("item-id") );
      this.render(true);      
    });

  }
  
  /* -------------------------------------------- */
  /** @override */
  setPosition(options = {}) {
    const position = super.setPosition(options);
    const sheetBody = this.element.find(".sheet-body");
    const bodyHeight = position.height - 192;
    sheetBody.css("height", bodyHeight);
    return position;
  }

  /* -------------------------------------------- */
  async _onDropItem(event, dragData) {
    let item = await PegasusUtility.searchItem( dragData)
    this.actor.preprocessItem( event, item, true )
    super._onDropItem(event, dragData)
  }

  /* -------------------------------------------- */
  /** @override */
  _updateObject(event, formData) {
    // Update the Actor
    return this.object.update(formData);
  }
}
